/*
DTMF_PelcoD_PTZ
Version 1.1  18 06 2017
Version 1.2  18 07 2017
 
Télécommande d'une camera PTZ via DTMF
Liaison  RS485 entre boitier et camera (protocole PelcoD).

Un code (ici 3950) permet de "réveiller" le système. Celui-ci
se remettra en veille automatiquement une minute après la
dernière commande. (variable "interval").

L'appui sur "A" remet en urgence le système en veille.

Si la camera est munie d'un zoom:
La fonction Zomm est assurée:
via protocole Pelco D 
via commutation de deux relais (Pins 4 et 5) 
via protocole Hitachi (Sortie D1)

Codes DTMF
2 Haut
8 Bas
4 Gauche
6 Droite
* Zoom ++
# Zoom --

Valeurs en sorties du MT8870
1,2,3,4,5,6,7,8,9
0 = 10
* = 11
# = 12
A = 13
B = 14
C = 15
D = 0


Plus d'informations sur mon site: http://f5mna.free.fr


*/

////////////////////////   DTMF   ////////////////////////////
int DTMF1 = 7;     // Sortie 1 MT8870 Pin 11
int DTMF2 = 8;     // Sortie 2 MT8870     12
int DTMF3 = 9;     // Sortie 3 MT8870     13
int DTMF4 = 10;    // Sortie 4 MT8870     14
int ConfDTMF = 11; // Confirmation de réception d'un code DTMF Pin 11
int Led = 12;      // Led
int DTMF = 0;      // Sommes sorties DTMF
int Cod = 0;       // Nbre de symbole DTMF pris en compte

int val;   // confirmation d'un code DTMF (ConfDTMF)
int valBt; // Bouton RAZ
int buttonState; // Prise en compte du changement d'état de la confirmation DTMF
int Buffer[25];    // Tableau mémorisation des codes entrant.
boolean Valid = false; // Validation de l'appel caméra

int Zplus = 5;  // Cde relais Zoom
int Zmoins = 4;

////// Tempo
 unsigned long currentMillis;
 long previousMillis = 0;    
 long interval = 60000;  //

//////////////////////  PELCO   ////////////////////////////
#include <RS485.h>
#include <SoftwareSerial.h>
const byte camNum = 0x01;
const byte tiltBas = 0x10;
const byte tiltHaut = 0x08;
const byte panDroite = 0x02;
const byte panGauche = 0x04;
const byte Zoomzero = 0x00;      // Stop Zoom
const byte Zoommoins = 0x20;     // Zoom + 0x01
const byte Zoomplus = 0x40;      // Zoom - 0x80
SoftwareSerial SerialRS = SoftwareSerial(2, 3);


void setup()
{
  Serial.begin (4800, SERIAL_8E1);
  SerialRS.begin(2400);
  
  pinMode (DTMF1, INPUT); // Sortie MT8870
  pinMode (DTMF2, INPUT); // Sortie MT8870
  pinMode (DTMF3, INPUT); // Sortie MT8870
  pinMode (DTMF4, INPUT); // Sortie MT8870
  pinMode (ConfDTMF, INPUT); // Validation code DTMF
  pinMode (Led, OUTPUT);     // Led
  pinMode (Zplus,OUTPUT);    // Relais Zoom
  pinMode (Zmoins,OUTPUT);   // Relais Zoom 

  digitalWrite (Zplus,LOW);
  digitalWrite (Zmoins,LOW); 
  
  buttonState = digitalRead(ConfDTMF);
  digitalWrite(Led, LOW);
  Serial.println ("DTMF PELCO");
}

void loop ()
{
  currentMillis = millis();  
  if(currentMillis - previousMillis > interval) {   //Tempo
  previousMillis = currentMillis; 
  digitalWrite(Led, LOW);
  Valid = false;
  }  
  val = digitalRead(ConfDTMF);                      //Confirmation DTMF
  if (val != buttonState) {                         // Modification de cette valeur
    if (val == HIGH) {                              // Si la valeur à changé.
      if  (Valid == false)
     { Cod++; }
      else
     { Cod = 1;}
      // Incrementation du compteur curseur (seconde ligne)
      
      if (digitalRead(DTMF4) == HIGH) DTMF = DTMF + 1;
      if (digitalRead(DTMF3) == HIGH) DTMF = DTMF + 2;
      if (digitalRead(DTMF2) == HIGH) DTMF = DTMF + 4;
      if (digitalRead(DTMF1) == HIGH) DTMF = DTMF + 8;
      
    if (DTMF == 3){
      Buffer[2] = 3;  // Doit être différent du second code de validation
      Buffer[3] = 3;  // Doit être différent du 3 eme code de validation
      Buffer[4] = 3;  // Doit être différent du 4 eme code de validation
      Cod = 1;
      //Valid = false;
      digitalWrite(Led, LOW);} 
         
      Buffer[Cod] = DTMF; 
      CodeCde();        
    }
   if (val == LOW)
     {
      digitalWrite (Zplus,LOW);
      digitalWrite (Zmoins,LOW); 
      Serial.println ("Stop");
      Stop();
    } }
  buttonState = val; // Confirmation de la validation DTMF (CM8870)
  DTMF = 0;  
}

void CodeCde()  // Les commandes PTZ
{
 
  if ((Buffer[1] == 3) and (Buffer[2] == 9) and (Buffer[3] == 5 )and (Buffer[4] == 10)) // 3950  Code de validation
  {
    Serial.println ("Validation");
    Valid = true;
    digitalWrite(Led, HIGH);
    Cod = 0;
    previousMillis = currentMillis; 
  }


   if ((Valid == true) and (DTMF == 13)) // 
  {
    Serial.println ("Veille");
    Valid = false;
    Buffer[2] = 3;  // Doit être différent du second code de validation
    Buffer[3] = 3;  // Doit être différent du 3 eme code de validation
    Buffer[4] = 3;  // Doit être différent du 4 eme code de validation
    Cod = 0;
    digitalWrite(Led, LOW);     
  }  


  if ((Valid == true) and (DTMF == 6)) // 
  {
    Serial.println ("Droite");
    sendData(0x01, Zoomzero, panDroite, 0x3F, 0x00);  //0x40 au lieu de 0x3F pour haute vitesse (turbo)
    previousMillis = currentMillis; 
  }  
 
  if ((Valid == true) and (DTMF == 4)) 
  {
    Serial.println ("Gauche");
    sendData(0x01, Zoomzero, panGauche, 0x3F, 0x00);  //0x40 au lieu de 0x3F pour haute vitesse (turbo)
    previousMillis = currentMillis;
  } 
 
  if ((Valid == true) and (DTMF == 2)) // 
  {
    Serial.println ("Haut");
    sendData(0x01, Zoomzero, tiltHaut, 0x00, 0x3F);
     previousMillis = currentMillis; 
  } 
 
  if ((Valid == true) and (DTMF == 8)) 
  {
    Serial.println ("Bas");
    sendData(0x01, Zoomzero, tiltBas, 0x00, 0x3F);
    previousMillis = currentMillis;
  }

  if ((Valid == true) and (DTMF == 11))
  {
    Serial.println ("Zoom ++");
    Serial.print (":WFCBB9B");  
    digitalWrite (Zplus,HIGH);
    sendData(0x01, Zoomzero, Zoomplus, 0x00, 0x00);
    previousMillis = currentMillis;
  }  

 if ((Valid == true) and (DTMF == 12)) 
  {
    Serial.println ("Zoom --");
    Serial.print (":WFCBB99"); 
    digitalWrite (Zmoins,HIGH); 
    sendData(0x01, Zoomzero, Zoommoins, 0x00, 0x00);
    previousMillis = currentMillis; 
  }  
}

void Stop ()
{
  byte dataVal[7] = {0xFF, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01};
  for (int i = 0; i < 7; i++)
  {
   SerialRS.write(dataVal[i]);  //envoyer octets "stop" //////////////// sendData (0xFF, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01);
  }
  Serial.print (":WFCBBFE");     //stop 
  Serial.println (":WFCBBA8");   //Auto focus on
  delay (300);                   //Attente mise au point
}


void sendData(byte camNum, byte command_1, byte command_2, byte panSpeed, byte tiltSpeed) //Envoi Commande avec checksum.
{
  int modSum = 0;                 //RAZ checksum.
  byte dataVal[6] = {0xFF, camNum, command_1, command_2, panSpeed, tiltSpeed}; // commandes Pelco-D
  for (int i = 0; i < 6; i++)     //envoi les données de la commande.
  {
    SerialRS.write(dataVal[i]);   //
    if (i > 0)                    // checksum commence à compter à partir du deuxième octet (Saute la première valeur 0xFF)
    modSum += dataVal[i];         // somme octets pour calcul checksum (Addition composée:  x += y équivaut à l'expresison x = x+y). 
  }
  modSum %= 100;                  // Calcul checksum. 
  SerialRS.write(modSum);         // envoi checksum.
}
   
 
 
